@@ -92,12 +92,14 @@ func (e *encodeGen) tuple(s *Struct) {
 		if !e.p.ok() {
 			return
 		}
-		anField := s.Fields[i].HasTagPart("allownil") && s.Fields[i].FieldElem.AllowNil()
+		fieldElem := s.Fields[i].FieldElem
+		anField := s.Fields[i].HasTagPart("allownil") && fieldElem.AllowNil()
 		if anField {
-			e.p.printf("\nif %s { // allownil: if nil", s.Fields[i].FieldElem.IfZeroExpr())
+			e.p.printf("\nif %s { // allownil: if nil", fieldElem.IfZeroExpr())
 			e.p.printf("\nerr = en.WriteNil(); if err != nil { return; }")
 			e.p.printf("\n} else {")
 		}
+		SetIsAllowNil(fieldElem, anField)
 		e.ctx.PushString(s.Fields[i].FieldName)
 		next(e, s.Fields[i].FieldElem)
 		e.ctx.Pop()
@@ -129,12 +131,13 @@ func (e *encodeGen) structmap(s *Struct) {
 	}
 
 	omitempty := s.AnyHasTagPart("omitempty")
+	omitzero := s.AnyHasTagPart("omitzero")
 	var fieldNVar string
-	if omitempty {
+	if omitempty || omitzero {
 
 		fieldNVar = oeIdentPrefix + "Len"
 
-		e.p.printf("\n// omitempty: check for empty values")
+		e.p.printf("\n// check for omitted fields")
 		e.p.printf("\n%s := uint32(%d)", fieldNVar, nfields)
 		e.p.printf("\n%s", bm.typeDecl())
 		e.p.printf("\n_ = %s", bm.varname)
@@ -147,6 +150,11 @@ func (e *encodeGen) structmap(s *Struct) {
 				e.p.printf("\n%s--", fieldNVar)
 				e.p.printf("\n%s", bm.setStmt(i))
 				e.p.printf("\n}")
+			} else if sf.HasTagPart("omitzero") {
+				e.p.printf("\nif %s.IsZero() {", sf.FieldElem.Varname())
+				e.p.printf("\n%s--", fieldNVar)
+				e.p.printf("\n%s", bm.setStmt(i))
+				e.p.printf("\n}")
 			}
 		}
 
@@ -164,11 +172,12 @@ func (e *encodeGen) structmap(s *Struct) {
 
 	} else {
 
-		// non-omitempty version
+		// non-omit version
 		data = msgp.AppendMapHeader(nil, uint32(nfields))
 		e.p.printf("\n// map header, size %d", nfields)
 		e.Fuse(data)
 		if len(s.Fields) == 0 {
+			e.p.printf("\n_ = %s", s.vname)
 			e.fuseHook()
 		}
 
@@ -179,23 +188,26 @@ func (e *encodeGen) structmap(s *Struct) {
 			return
 		}
 
-		// if field is omitempty, wrap with if statement based on the emptymask
-		oeField := omitempty && s.Fields[i].HasTagPart("omitempty") && s.Fields[i].FieldElem.IfZeroExpr() != ""
+		// if field is omitempty or omitzero, wrap with if statement based on the emptymask
+		oeField := (omitempty || omitzero) &&
+			((s.Fields[i].HasTagPart("omitempty") && s.Fields[i].FieldElem.IfZeroExpr() != "") ||
+				s.Fields[i].HasTagPart("omitzero"))
 		if oeField {
-			e.p.printf("\nif %s == 0 { // if not empty", bm.readExpr(i))
+			e.p.printf("\nif %s == 0 { // if not omitted", bm.readExpr(i))
 		}
 
 		data = msgp.AppendString(nil, s.Fields[i].FieldTag)
 		e.p.printf("\n// write %q", s.Fields[i].FieldTag)
 		e.Fuse(data)
 		e.fuseHook()
-
-		anField := !oeField && s.Fields[i].HasTagPart("allownil") && s.Fields[i].FieldElem.AllowNil()
+		fieldElem := s.Fields[i].FieldElem
+		anField := !oeField && s.Fields[i].HasTagPart("allownil") && fieldElem.AllowNil()
 		if anField {
 			e.p.printf("\nif %s { // allownil: if nil", s.Fields[i].FieldElem.IfZeroExpr())
 			e.p.printf("\nerr = en.WriteNil(); if err != nil { return; }")
 			e.p.printf("\n} else {")
 		}
+		SetIsAllowNil(fieldElem, anField)
 
 		e.ctx.PushString(s.Fields[i].FieldName)
 		next(e, s.Fields[i].FieldElem)
