@@ -1,12 +1,12 @@
 package gen
 
 import (
+	"bytes"
 	"fmt"
 	"io"
 )
 
 const (
-	errcheck    = "\nif err != nil { return }"
 	lenAsUint32 = "uint32(len(%s))"
 	literalFmt  = "%s"
 	intFmt      = "%d"
@@ -62,25 +62,6 @@ func (m Method) String() string {
 	}
 }
 
-func strtoMeth(s string) Method {
-	switch s {
-	case "encode":
-		return Encode
-	case "decode":
-		return Decode
-	case "marshal":
-		return Marshal
-	case "unmarshal":
-		return Unmarshal
-	case "size":
-		return Size
-	case "test":
-		return Test
-	default:
-		return 0
-	}
-}
-
 const (
 	Decode      Method                       = 1 << iota // msgp.Decodable
 	Encode                                               // msgp.Encodable
@@ -158,7 +139,14 @@ func (p *Printer) ApplyDirective(pass Method, t TransformPass) {
 // Print prints an Elem.
 func (p *Printer) Print(e Elem) error {
 	for _, g := range p.gens {
+		// Elem.SetVarname() is called before the Print() step in parse.FileSet.PrintTo().
+		// Elem.SetVarname() generates identifiers as it walks the Elem. This can cause
+		// collisions between idents created during SetVarname and idents created during Print,
+		// hence the separate prefixes.
+		resetIdent("zb")
 		err := g.Execute(e)
+		resetIdent("za")
+
 		if err != nil {
 			return err
 		}
@@ -166,6 +154,49 @@ func (p *Printer) Print(e Elem) error {
 	return nil
 }
 
+type contextItem interface {
+	Arg() string
+}
+
+type contextString string
+
+func (c contextString) Arg() string {
+	return fmt.Sprintf("%q", c)
+}
+
+type contextVar string
+
+func (c contextVar) Arg() string {
+	return string(c)
+}
+
+type Context struct {
+	path []contextItem
+}
+
+func (c *Context) PushString(s string) {
+	c.path = append(c.path, contextString(s))
+}
+
+func (c *Context) PushVar(s string) {
+	c.path = append(c.path, contextVar(s))
+}
+
+func (c *Context) Pop() {
+	c.path = c.path[:len(c.path)-1]
+}
+
+func (c *Context) ArgsStr() string {
+	var out string
+	for idx, p := range c.path {
+		if idx > 0 {
+			out += ", "
+		}
+		out += p.Arg()
+	}
+	return out
+}
+
 // generator is the interface through
 // which code is generated.
 type generator interface {
@@ -287,18 +318,18 @@ func (p *printer) declare(name string, typ string) {
 
 // does:
 //
-// if m != nil && size > 0 {
-//     m = make(type, size)
-// } else if len(m) > 0 {
-//     for key, _ := range m { delete(m, key) }
-// }
+//	if m == nil {
+//	    m = make(type, size)
+//	} else if len(m) > 0 {
 //
+//	    for key := range m { delete(m, key) }
+//	}
 func (p *printer) resizeMap(size string, m *Map) {
 	vn := m.Varname()
 	if !p.ok() {
 		return
 	}
-	p.printf("\nif %s == nil && %s > 0 {", vn, size)
+	p.printf("\nif %s == nil {", vn)
 	p.printf("\n%s = make(%s, %s)", vn, m.TypeName(), size)
 	p.printf("\n} else if len(%s) > 0 {", vn)
 	p.clearMap(vn)
@@ -315,13 +346,27 @@ func (p *printer) mapAssign(m *Map) {
 
 // clear map keys
 func (p *printer) clearMap(name string) {
-	p.printf("\nfor key, _ := range %[1]s { delete(%[1]s, key) }", name)
+	p.printf("\nfor key := range %[1]s { delete(%[1]s, key) }", name)
+}
+
+func (p *printer) wrapErrCheck(ctx string) {
+	p.print("\nif err != nil {")
+	p.printf("\nerr = msgp.WrapError(err, %s)", ctx)
+	p.printf("\nreturn")
+	p.print("\n}")
 }
 
 func (p *printer) resizeSlice(size string, s *Slice) {
 	p.printf("\nif cap(%[1]s) >= int(%[2]s) { %[1]s = (%[1]s)[:%[2]s] } else { %[1]s = make(%[3]s, %[2]s) }", s.Varname(), size, s.TypeName())
 }
 
+// resizeSliceNoNil will resize a slice and will not allow nil slices.
+func (p *printer) resizeSliceNoNil(size string, s *Slice) {
+	p.printf("\nif %[1]s != nil && cap(%[1]s) >= int(%[2]s) {", s.Varname(), size)
+	p.printf("\n%[1]s = (%[1]s)[:%[2]s]", s.Varname(), size)
+	p.printf("\n} else { %[1]s = make(%[3]s, %[2]s) }", s.Varname(), size, s.TypeName())
+}
+
 func (p *printer) arrayCheck(want string, got string) {
 	p.printf("\nif %[1]s != %[2]s { err = msgp.ArrayError{Wanted: %[2]s, Got: %[1]s}; return }", got, want)
 }
@@ -330,14 +375,15 @@ func (p *printer) closeblock() { p.print("\n}") }
 
 // does:
 //
-// for idx := range iter {
-//     {{generate inner}}
-// }
-//
-func (p *printer) rangeBlock(idx string, iter string, t traversal, inner Elem) {
+//	for idx := range iter {
+//	    {{generate inner}}
+//	}
+func (p *printer) rangeBlock(ctx *Context, idx string, iter string, t traversal, inner Elem) {
+	ctx.PushVar(idx)
 	p.printf("\n for %s := range %s {", idx, iter)
 	next(t, inner)
 	p.closeblock()
+	ctx.Pop()
 }
 
 func (p *printer) nakedReturn() {
@@ -374,3 +420,82 @@ func (p *printer) ok() bool { return p.err == nil }
 func tobaseConvert(b *BaseElem) string {
 	return b.ToBase() + "(" + b.Varname() + ")"
 }
+
+func (p *printer) varWriteMapHeader(receiver string, sizeVarname string, maxSize int) {
+	if maxSize <= 15 {
+		p.printf("\nerr = %s.Append(0x80 | uint8(%s))", receiver, sizeVarname)
+	} else {
+		p.printf("\nerr = %s.WriteMapHeader(%s)", receiver, sizeVarname)
+	}
+}
+
+func (p *printer) varAppendMapHeader(sliceVarname string, sizeVarname string, maxSize int) {
+	if maxSize <= 15 {
+		p.printf("\n%s = append(%s, 0x80 | uint8(%s))", sliceVarname, sliceVarname, sizeVarname)
+	} else {
+		p.printf("\n%s = msgp.AppendMapHeader(%s, %s)", sliceVarname, sliceVarname, sizeVarname)
+	}
+}
+
+// bmask is a bitmask of a the specified number of bits
+type bmask struct {
+	bitlen  int
+	varname string
+}
+
+// typeDecl returns the variable declaration as a var statement
+func (b *bmask) typeDecl() string {
+	return fmt.Sprintf("var %s %s /* %d bits */", b.varname, b.typeName(), b.bitlen)
+}
+
+// typeName returns the type, e.g. "uint8" or "[2]uint64"
+func (b *bmask) typeName() string {
+	if b.bitlen <= 8 {
+		return "uint8"
+	}
+	if b.bitlen <= 16 {
+		return "uint16"
+	}
+	if b.bitlen <= 32 {
+		return "uint32"
+	}
+	if b.bitlen <= 64 {
+		return "uint64"
+	}
+
+	return fmt.Sprintf("[%d]uint64", (b.bitlen+64-1)/64)
+}
+
+// readExpr returns the expression to read from a position in the bitmask.
+// Compare ==0 for false or !=0 for true.
+func (b *bmask) readExpr(bitoffset int) string {
+	if bitoffset < 0 || bitoffset >= b.bitlen {
+		panic(fmt.Errorf("bitoffset %d out of range for bitlen %d", bitoffset, b.bitlen))
+	}
+
+	var buf bytes.Buffer
+	buf.Grow(len(b.varname) + 16)
+	buf.WriteByte('(')
+	buf.WriteString(b.varname)
+	if b.bitlen > 64 {
+		fmt.Fprintf(&buf, "[%d]", (bitoffset / 64))
+	}
+	buf.WriteByte('&')
+	fmt.Fprintf(&buf, "0x%X", (uint64(1) << (uint64(bitoffset) % 64)))
+	buf.WriteByte(')')
+
+	return buf.String()
+}
+
+// setStmt returns the statement to set the specified bit in the bitmask.
+func (b *bmask) setStmt(bitoffset int) string {
+	var buf bytes.Buffer
+	buf.Grow(len(b.varname) + 16)
+	buf.WriteString(b.varname)
+	if b.bitlen > 64 {
+		fmt.Fprintf(&buf, "[%d]", (bitoffset / 64))
+	}
+	fmt.Fprintf(&buf, " |= 0x%X", (uint64(1) << (uint64(bitoffset) % 64)))
+
+	return buf.String()
+}
